### Data preparation examples ###
library(dismo)
library(ggmap)

# we can download the data from the GBIF, obviously. However....
# acaule = gbif("solanum", "acaule*", geo=FALSE)

# ...for this example, we are using a data frame available in the package dismo
data(acaule)

# let's see how many records/columns (attributes)
dim(acaule)

# columns (attributes) names
colnames(acaule)

# store in a new data frame the records which have coordinates only
acgeo <- subset(acaule, !is.na(lon) & !is.na(lat))

# let's see how many records/columns (attributes)
dim(acgeo)

# let's see some records/columns (attributes)
acgeo[1:4, c(1:5,7:10)]

# we need a new library to plot maps
library(maptools)

# let's load a simple country boredr world map
data(wrld_simpl)

# and plot it in the area of occurrence of S. acaule (South America)
plot(wrld_simpl, xlim=c(-80,70), ylim=c(-60,10), axes=TRUE, col="light yellow")
plot(wrld_simpl, axes=TRUE, col="light yellow")
# plot points
points(acgeo$lon, acgeo$lat, col='orange', pch=20, cex=0.75) 
points(acgeo$lon, acgeo$lat, col='red', cex=0.75)
box()

lonzero = subset(acgeo, lon==0)
# show all records, only the first 13 columns
lonzero[, 1:13]
# get rid of duplicates
dups <- duplicated(lonzero[, 1:10])
lonzero <- lonzero[dups, ]
# show records again
lonzero[,1:13]

# let's get rid of duplicates from acgeo
dups2 <- duplicated(acgeo[, c('lon', 'lat')])
sum(dups2)
acg <- acgeo[!dups2, ]

# we know that the species is endemic of Latin America.
# Thus, let's check whether there are points which fall poutside its range
# To do this, we simply check which point have latitude and longitude
# higher than 0 (Latin America has long -10 - -60 and lat -40 - -70).
i <- acg$lon > 0 & acg$lat > 0
i
# let's give a look to one of them
acg[379, ]

# We can imagine these are errors in writing the coordinates
# and the sign - was omitted / lost.
# Thus, we simply traslate the points, by multiplying the coordinates
# by -1. This is done only to the records for which i is TRUE
acg$lon[i] <- -1 * acg$lon[i]
acg$lat[i] <- -1 * acg$lat[i]
# Plus, we get rid of all the point in acg which have coordinates which
# falls outside the known distribution of the species, since they are
# surely erroneous (see the point in Brasil, and those with longitude 0)
# but we cannot relocate them with certainty
acg <- acg[acg$lon < -50 & acg$lat > -50, ]

# and plot again everything
plot(wrld_simpl, xlim=c(-80,70), ylim=c(-60,10), axes=TRUE, col="light yellow")
points(acg$lon, acg$lat, col='orange', pch=20, cex=0.75) 
points(acg$lon, acg$lat, col='red', cex=0.75)
box()

# let's view it bigger
plot(wrld_simpl, xlim=c(-80,-45), ylim=c(-35,10), axes=TRUE, col="light yellow")
points(acg$lon, acg$lat, col='orange', pch=20, cex=0.75) 
points(acg$lon, acg$lat, col='red', cex=0.75)
box()


# let's go for some spatial query, to further check the data
# let's load the library we need
library(sp)
# now we use the function coordinates() to transform our data frame in 
# a SpatialPointsDataFrame (a different class of objects), by specifying
# the two columns which contain latitude and longitude
?coordinates
coordinates(acg) <- ~lon+lat
# then we set the coordinate reference system (CRS) of it
# to the projection and datum of wrld_simpl
?crs
crs(acg) <- crs(wrld_simpl)
# now we make some spatial queries.
# First, we create a novel oject, which contains lat and lon from acg,
# plus all the info in wrld_simpl. We use the function over().
# it works retrieving, at the spatial locations of object x (acg), the 
# indexes or attributes from spatial object y (wrld_simpl)
?over
ovr <- over(acg, wrld_simpl)
# the object ovr is a data frame. We can see its first records
head(ovr)
# what we need is the name of the country
cntr <- ovr$NAME
cntr

# now first query: are there records without country? The answer is no,
# but better safe than sorry
i <- which(is.na(cntr))
i
# in fact, 0

# now the secodn query, i.e. which records have a country in the original 
# data which does not match the country in wrld_simpl
j <- which(cntr != acg$country)
j
# and here we have an array of 4 records

# now let's plot the points of acg, then the country borders from 
# wrld_simpl. Then we plot in red the points with mismatch between
# coordinates and country name. 
plot(acg)
plot(wrld_simpl, add=T, border='blue', lwd=2)
points(acg[j, ], col='red', pch=20, cex=2)
# In this case, the mismatch is probably due to the closeness of
# the points to the borders of the countries. The wrld_simpl is not very 
# precise, and is used for siumple analysis. Thus, probably the points
# are correct. Anyway, this is not a big deal in this dataset.

# and now some georeferencing....
# let's store in the georef data frame all the records with no lat and lon,
# but with information on the locality of collection / observation
georef <- subset(acaule, (is.na(lon) | is.na(lat)) & ! is.na(locality) )
# let's see what we got
dim(georef)
# some detail
georef[1:3,1:13]
# let's see which columns I have in the data frame
colnames(georef)
# let's see the content of the column cloc for record 1
georef$cloc[1]
# let's see the content of the column cloc (25) for the first 3 records
georef[1:3,25]

# let's try to retrieve coords for one point by using google
# to do this, you need to obtain a google API key, and set it in R
?geocode
b <- try( geocode(georef$cloc[1]) )
b


### Sampling bias
# since often there is oversamplig in some partions of the distribution range
# we do some resampling, retaining one (or more) occurrences per cell in a grid
# create a RasterLayer with the extent of acg
r <- raster(acg)
# set the resolution of the cells to (for example) 1 degree
res(r) <- 1
# expand (extend) the extent of the RasterLayer a little
r <- extend(r, extent(r)+1)
# resample
acsel <- gridSample(acg, r, n=1)
# to understand the command gridSample
?gridSample
# view the the result
p <- rasterToPolygons(r)
plot(p, border='gray')
points(acg)
# resampled points in red
points(acsel, cex=1, col='red', pch='x')


#### FURTHER EXERCISES ####

library(dismo)
library(ggmap)

# 1) Use the gbif function to download records for Crocus biflorus 
# (or another species of your preference, try to get one with between 
# 10 and 100 records). Use option ”geo=FALSE” to also get records with 
# no (numerical) georeference
biflorus = gbif("crocus", "biflorus*", geo=FALSE)

# 2) Summarize the data: how many records are there, how many have 
# coordinates, how many records without coordinates have a textual
# georeference (locality description)?
colnames(biflorus)
dim(biflorus)
biflorus_geo <- subset(biflorus, !is.na(lon) & !is.na(lat))
dim(biflorus_geo)
biflorus_non_geo <- subset(biflorus, is.na(lon) | is.na(lat))
dim(biflorus_non_geo)

# 3) Use the ’geocode’ function to georeference the first 10 records
# without coordinates
biflorus_non_geo[1:10,15]
library(dplyr)
biflorus_re_geo <- biflorus_non_geo %>% slice(1:10)
dim(biflorus_re_geo)
colnames(biflorus_re_geo)

# lat [80]
# lon [84]

b <- try( geocode(biflorus_non_geo$cloc[1]) )
b
# data.frame[row_number, column_number] = new_value
biflorus_re_geo[1,80]=b$lat
biflorus_re_geo[1,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[2]) )
b
biflorus_re_geo[2,80]=b$lat
biflorus_re_geo[2,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[3]) )
b
biflorus_re_geo[3,80]=b$lat
biflorus_re_geo[3,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[4]) )
b
biflorus_re_geo[4,80]=b$lat
biflorus_re_geo[4,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[5]) )
b
biflorus_re_geo[5,80]=b$lat
biflorus_re_geo[5,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[6]) )
b
biflorus_re_geo[6,80]=b$lat
biflorus_re_geo[6,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[7]) )
b
biflorus_re_geo[7,80]=b$lat
biflorus_re_geo[7,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[8]) )
b
biflorus_re_geo[8,80]=b$lat
biflorus_re_geo[8,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[9]) )
b
biflorus_re_geo[9,80]=b$lat
biflorus_re_geo[9,84]=b$lon
b <- try( geocode(biflorus_non_geo$cloc[10]) )
b
biflorus_re_geo[10,80]=b$lat
biflorus_re_geo[10,84]=b$lon

# let's see whether it worked or not
biflorus_re_geo[1:10,c(80,84)]

# 4) Make a simple map of all the records, using a color and symbol to distinguish between the coordinates from gbif and the ones returned by Google (via the geocode function). Use ’gmap’ to create a basemap.
library(maptools)

data(wrld_simpl)

plot(wrld_simpl, xlim=c(0,60), ylim=c(30,65), axes=TRUE, col="light yellow")

# plot points
points(biflorus_geo$lon, biflorus_geo$lat, col='orange', pch=20, cex=0.75) 

points(biflorus_re_geo$lon, biflorus_re_geo$lat, col='blue', pch=20, cex=0.75) 

box()

